@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2016 The ANTLR Project. All rights reserved.
+ * Copyright (c) 2012-2017 The ANTLR Project. All rights reserved.
  * Use of this file is governed by the BSD 3-clause license that
  * can be found in the LICENSE.txt file in the project root.
  */
@@ -31,6 +31,7 @@ import java.util.ArrayList;
 import java.util.List;
 
 import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertNotEquals;
 
 public class TestCodeGeneration extends BaseJavaToolTest {
 	@Before
@@ -52,6 +53,24 @@ public class TestCodeGeneration extends BaseJavaToolTest {
 		}
 	}
 
+	@Test public void AssignTokenNamesToStringLiteralsInGeneratedParserRuleContexts() throws Exception {
+		String g =
+			"grammar T;\n" +
+			"root: 't1';\n" +
+			"Token: 't1';";
+		List<String> evals = getEvalInfoForString(g, "() { return getToken(");
+		assertNotEquals(0, evals.size());
+	}
+
+	@Test public void AssignTokenNamesToStringLiteralArraysInGeneratedParserRuleContexts() throws Exception {
+		String g =
+			"grammar T;\n" +
+				"root: 't1' 't1';\n" +
+				"Token: 't1';";
+		List<String> evals = getEvalInfoForString(g, "() { return getTokens(");
+		assertNotEquals(0, evals.size());
+	}
+
 	/** Add tags around each attribute/template/value write */
 	public static class DebugInterpreter extends Interpreter {
 		List<String> evals = new ArrayList<String>();
