@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2016 The ANTLR Project. All rights reserved.
+ * Copyright (c) 2012-2017 The ANTLR Project. All rights reserved.
  * Use of this file is governed by the BSD 3-clause license that
  * can be found in the LICENSE.txt file in the project root.
  */
@@ -146,6 +146,22 @@ public class TokenVocabParser {
 		// files are generated (in the base, not relative to the input
 		// location.)
 		f = new File(g.tool.outputDirectory, vocabName + CodeGenerator.VOCAB_FILE_EXTENSION);
-		return f;
+		if ( f.exists() ) {
+			return f;
+		}
+		
+		// Still not found? Use the grammar's subfolder then.
+		String fileDirectory;
+
+		if (g.fileName.lastIndexOf(File.separatorChar) == -1) {
+			// No path is included in the file name, so make the file
+			// directory the same as the parent grammar (which might still be just ""
+			// but when it is not, we will write the file in the correct place.
+			fileDirectory = ".";
+		}
+		else {
+			fileDirectory = g.fileName.substring(0, g.fileName.lastIndexOf(File.separatorChar));
+		}
+		return new File(fileDirectory, vocabName + CodeGenerator.VOCAB_FILE_EXTENSION);
 	}
 }
